import ctypes
from ctypes import *

# load PACSDK SDK
pac_api = ctypes.CDLL('.\\PACSDK.dll')

# 
PAC_REMOTE_IO = lambda iAddress: 0x1000 | iAddress

# 
pac_api.uart_Open.argtypes = [c_char_p]
pac_api.uart_Open.restype = c_void_p

pac_api.uart_Close.argtypes = [c_void_p]
pac_api.uart_Close.restype = c_bool

pac_api.uart_SendCmd.argtypes = [c_void_p, c_char_p, POINTER(c_char)]
pac_api.uart_SendCmd.restype = c_bool

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong

def get_last_error():
  error_code = pac_api.pac_GetLastError()
  error_message = create_string_buffer(64)
  pac_api.pac_GetErrorMessage(error_code, error_message)
  return error_message.value.decode()


def main():
  print("This demo is for remote 87K and 7K modules.")

  connection_string = input("Input com's connectionstring(like 'COM2,9600,N,8,1'): ")
  
  # open COM port
  hPort = pac_api.uart_Open(c_char_p(connection_string.encode('utf-8')))
  if hPort == ctypes.c_void_p(-1).value:
      print(f"Open Error: {get_last_error()}")
      return

  while True:
      command = input("Input command (type 'q' to exit): ")
      if command in ["q", "quit"]:
          break
      
      response = create_string_buffer(20)
      result = pac_api.uart_SendCmd(hPort, c_char_p(command.encode('utf-8')), response)
      
      if not result:
          print(f"Error: {get_last_error()}")
      else:
          print(f"Response: {response.value.decode()}")

  # close COM port
  if not pac_api.uart_Close(hPort):
      print(f"Close Error: {get_last_error()}")

if __name__ == "__main__":
  main()
