import tkinter as tk
from tkinter import ttk, messagebox
import ctypes
from ctypes import *

# Load the pacsdk.dll
pac_api = ctypes.CDLL('./PACSDK.dll')

PAC_REMOTE_IO = lambda iAddress: 0x1000 | iAddress
    
# Define UART and AI Read functions (prototypes for the pacsdk functions)
pac_api.uart_Open.argtypes = [c_char_p]
pac_api.uart_Open.restype = c_void_p

pac_api.uart_Close.argtypes = [c_void_p]

pac_api.pac_SetPWMDuty.argtypes = [c_void_p, c_int, c_short, c_float]
pac_api.pac_SetPWMDuty.restype = c_bool

pac_api.pac_GetPWMDuty.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_float)]
pac_api.pac_GetPWMDuty.restype = c_bool

pac_api.pac_SetPWMFrequency.argtypes = [c_void_p, c_int, c_short, c_ulong]
pac_api.pac_SetPWMFrequency.restype = c_bool

pac_api.pac_GetPWMFrequency.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_ulong)]
pac_api.pac_GetPWMFrequency.restype = c_bool

pac_api.pac_SetPWMMode.argtypes = [c_void_p, c_int, c_short, c_long]
pac_api.pac_SetPWMMode.restype = c_bool

pac_api.pac_GetPWMMode.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_long)]
pac_api.pac_GetPWMMode.restype = c_bool

pac_api.pac_SetPWMStart.argtypes = [c_void_p, c_int, c_short]
pac_api.pac_SetPWMStart.restype = c_bool

pac_api.pac_SavePWMConfig.argtypes = [c_void_p, c_int]
pac_api.pac_SavePWMConfig.restype = c_bool

pac_api.pac_GetPWMDIOStatus.argtypes = [c_void_p, c_int, ctypes.POINTER(c_ubyte), ctypes.POINTER(c_ubyte)]
pac_api.pac_GetPWMDIOStatus.restype = c_bool

pac_api.pac_SetPWMPulseCount.argtypes = [c_void_p, c_int, c_short, c_long]
pac_api.pac_SetPWMPulseCount.restype = c_bool

pac_api.pac_GetPWMPulseCount.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_long)]
pac_api.pac_GetPWMPulseCount.restype = c_bool

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong


# Tkinter GUI setup
class Form1(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("7K87K PWM")
        self.geometry("850x600")  # Increased window size
        
        base_width = 100

        # Calculate adjusted x-positions with spacing
        x_port_label = 12
        x_port_entry = x_port_label + base_width + 10  # Add spacing between label and entry

        x_baudrate_label = x_port_entry + base_width + 20  # Add more spacing for wider "Baud rate" label
        x_baudrate_entry = x_baudrate_label + base_width + 10

        x_address_label = x_baudrate_entry + base_width + 20  # Add more spacing for wider "Address" label
        x_address_entry = x_address_label + base_width + 10
        
        # Components
        self.txtPort = tk.Entry(self)
        self.txtPort.place(x=x_port_entry, y=12, width=70, height=30)
        self.txtPort.insert(0, "5")

        self.label17 = tk.Label(self, text="COM Port")
        self.label17.place(x=x_port_label, y=12, width=80, height=30)

        self.txtBaudrate = tk.Entry(self)
        self.txtBaudrate.place(x=x_baudrate_entry, y=12, width=100, height=30)
        self.txtBaudrate.insert(0, "9600")

        self.label16 = tk.Label(self, text="Baud rate")
        self.label16.place(x=x_baudrate_label, y=12, width=80, height=30)

        self.txtAddress = tk.Entry(self)
        self.txtAddress.place(x=x_address_entry, y=12, width=100, height=30)
        self.txtAddress.insert(0, "1")

        self.label15 = tk.Label(self, text="Address")
        self.label15.place(x=x_address_label, y=12, width=80, height=30)
       
        
        self.btnClose = tk.Button(self, text="Close COM", command=self.btnClose_Click)
        self.btnClose.place(x=136, y=60, width=120, height=35)  # Increased width and height

        self.btnOpen = tk.Button(self, text="Open COM", command=self.btnOpen_Click)
        self.btnOpen.place(x=12, y=60, width=120, height=35)  # Increased width and height
       
        self.btnRead = tk.Button(self, text="Read PWM Activated and DI", command=self.btnRead_Click)
        self.btnRead.place(x=435, y=460, width=240, height=35)  # Increased width and height

        self.btnSavePWM = tk.Button(self, text="Save PWM Configurations", command=self.btnSavePWM_Click)
        self.btnSavePWM.place(x=36, y=460, width=240, height=35)  # Increased width and height


         # Assuming a base width of each widget (adjust as needed)
        base_width = 40
        base_height = 35  # Assuming consistent height for most widgets

        # Calculate adjusted x-positions with spacing
        x_left_column = 60  # Start of left column (Frequency label)
        x_center_column = x_left_column + base_width + 50  # Center column (Duty label)
        x_center_column_1=x_center_column+ base_width + 20  # Center column (pwm pulse mode label)
        x_center_column_2= x_center_column_1 + base_width + 100  # Center column (Burst count label)
        x_center_column_3= x_center_column_2 + base_width + 50  # Center column (Apply PWM Settings)
        x_right_column_1 = x_center_column_3 + base_width + 180  # Right column (DI Status)
        x_right_column_2 = x_right_column_1 + base_width + 60  # Right column (PWM Status)

        # Calculate starting y-position for the first row
        y_pos = 125
        y_2ed_pos = y_pos+40

        # Create and place widgets

        # Top row labels
        self.label3 = tk.Label(self, text="PWM Pulse Mode")
        self.label3.place(x=x_center_column_1-20, y=y_pos, width=170, height=base_height)

        self.label5 = tk.Label(self, text="Frequency (Hz)")
        self.label5.place(x=x_left_column-45, y=y_pos, width=150, height=base_height)

        self.label7 = tk.Label(self, text="Duty")
        self.label7.place(x=x_center_column, y=y_pos, width=50, height=base_height)

        # "Burst Count" label
        self.label19 = tk.Label(self, text="Burst Count")
        self.label19.place(x=x_center_column_2, y=y_pos, width=100, height=base_height)

        self.label19 = tk.Label(self, text="Start PWM")
        self.label19.place(x=x_center_column_3+20, y=y_pos, width=100, height=base_height)

        # Channel labels
        self.channel = [tk.Label(self, text=f"CH{i}:") for i in range(8)]
        for i, lb in enumerate(self.channel):
            lb.place(x=1, y=y_2ed_pos + i * 35, width=40, height=base_height)

        # DI Status Labels
        self.lbDI = [tk.Label(self, text="OFF") for _ in range(8)]
        for i, lb in enumerate(self.lbDI):
            lb.place(x=x_right_column_2 + 10, y=y_2ed_pos + i * 35, width=40, height=base_height)  # Added spacing

        self.label18 = tk.Label(self, text="DI Status")
        self.label18.place(x=x_right_column_2, y=y_pos, width=80, height=base_height)

        # PWM Status Labels
        self.pwm = [tk.Label(self, text="OFF") for _ in range(8)]
        for i, lb in enumerate(self.pwm):
            lb.place(x=x_right_column_1 + 10, y=y_2ed_pos + i * 35, width=40, height=base_height)  # Added spacing

        self.label20 = tk.Label(self, text="PWM Status")
        self.label20.place(x=x_right_column_1, y=y_pos, width=100, height=base_height)

        # Duty, Frequency, and Mode entries (adjusted positions)
        self.txtDuty = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtDuty):
            entry.place(x=x_center_column, y=y_2ed_pos + i * 35, width=50, height=base_height)
            entry.insert(0, "50")

        self.txtHz = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtHz):
            entry.place(x=x_left_column, y=y_2ed_pos + i * 35, width=80, height=base_height)
            entry.insert(0, "1000")

        self.txtCnt = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtCnt):
            entry.place(x=x_center_column_2, y=y_2ed_pos + i * 35, width=80, height=30)  # Increased width and height
            entry.insert(0, "10000")
            
        self.cboMode = [ttk.Combobox(self, values=["Burst Count", "Continuous"]) for _ in range(8)]
        for i, combo in enumerate(self.cboMode):
            combo.place(x=x_center_column_1, y=y_2ed_pos + i * 35, width=120, height=base_height)

        # Button and Checkbutton positions (adjusted)
        self.btnPWM = [tk.Button(self, text="Apply PWM Settings", command=lambda i=i: self.ConfigPWMSettings(i)) for i in range(8)]
        for i, btn in enumerate(self.btnPWM):
            btn.place(x=x_center_column_3, y=y_2ed_pos + i * 35, width=180, height=base_height)
        
        self.pwmStartBit = bytearray(8)
        self.pwmVar = [tk.IntVar() for _ in range(8)]
        #self.ckEnPWM = [tk.Checkbutton(self, command=lambda i=i: self.EnablePWM(i)) for i in range(8)]           
        self.ckEnPWM = [tk.Checkbutton(self, command=lambda i=i: self.EnablePWM(i), variable=self.pwmVar[i])for i in range(8)]
        for i, ck in enumerate(self.ckEnPWM):
            ck.place(x=x_center_column_3+180, y=y_2ed_pos + i * 35, width=30, height=30)  # Increased width and height            

    def btnOpen_Click(self):
        connStr = f"COM{self.txtPort.get()},{self.txtBaudrate.get()}"
        # Step1: Open COM port
        self.hPort = pac_api.uart_Open(connStr.encode('utf-8'))
        if self.hPort == c_void_p(-1).value:
            ec = pac_api.pac_GetLastError()
            messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")

    def btnClose_Click(self):
        pac_api.uart_Close(self.hPort)

    def btnRead_Click(self):
        self.ReadPWM_DIStatus()

    def btnSavePWM_Click(self):
        ret = pac_api.pac_SavePWMConfig(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())))
        if not ret:
            ec = pac_api.pac_GetLastError()
            messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")

    def ConfigPWMSettings(self, chIndex):
        hzArr = [entry.get() for entry in self.txtHz]
        dutyArr = [entry.get() for entry in self.txtDuty]
        cntArr = [entry.get() for entry in self.txtCnt]
        modeArr = [combo.current() for combo in self.cboMode]

        ret = pac_api.pac_SetPWMFrequency(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, int(hzArr[chIndex]))
        if not ret:
            ec = pac_api.pac_GetLastError()
            messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")
        else:
            pac_api.pac_SetPWMDuty(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, float(dutyArr[chIndex]))
            pac_api.pac_SetPWMPulseCount(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, int(cntArr[chIndex]))
            pac_api.pac_SetPWMMode(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, modeArr[chIndex])

            self.GetPWMConfig(chIndex)

    def GetPWMConfig(self, chIndex):
        hzArr = [entry.get() for entry in self.txtHz]
        dutyArr = [entry.get() for entry in self.txtDuty]
        cntArr = [entry.get() for entry in self.txtCnt]
        modeArr = [combo.get() for combo in self.cboMode]

        freq = c_ulong()
        ret = pac_api.pac_GetPWMFrequency(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, ctypes.byref(freq))
        if not ret:
            ec = pac_api.pac_GetLastError()
            messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")
        else:
            self.txtHz[chIndex].delete(0, tk.END)
            self.txtHz[chIndex].insert(0, freq.value)
            duty = c_float()
            ret = pac_api.pac_GetPWMDuty(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, ctypes.byref(duty))
            self.txtDuty[chIndex].delete(0, tk.END)
            self.txtDuty[chIndex].insert(0, f"{duty.value:.1f}")
            cnt = c_long()
            ret = pac_api.pac_GetPWMPulseCount(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, ctypes.byref(cnt))
            self.txtCnt[chIndex].delete(0, tk.END)
            self.txtCnt[chIndex].insert(0, cnt.value)

            mode = c_long()
            ret = pac_api.pac_GetPWMMode(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), chIndex, ctypes.byref(mode))
            self.cboMode[chIndex].set(mode.value)

    def ReadPWM_DIStatus(self):
        lbDI = self.lbDI
        lbPWM = self.pwm
        enPWM = self.ckEnPWM

        pwmBit = (c_ubyte * 8)()
        diBit = (c_ubyte * 8)()
        ret = pac_api.pac_GetPWMDIOStatus(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), pwmBit, diBit)
        if not ret:
            ec = pac_api.pac_GetLastError()
            messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")
        else:
            for i in range(8):
                if pwmBit[i] == 1:
                    lbPWM[i].config(text="ON", bg="red")
                    enPWM[i].select()
                else:
                    lbPWM[i].config(text="OFF", bg="gray")
                    enPWM[i].deselect()

                if diBit[i] == 1:
                    lbDI[i].config(text="ON", bg="green")
                else:
                    lbDI[i].config(text="OFF", bg="gray")

    def EnablePWM(self, chIndex):
          status = 0
          enBox = self.ckEnPWM[chIndex]
          
          #if enBox.var.get():  # Check if the checkbox is checked
          if self.pwmVar[chIndex].get():
              self.pwmStartBit[chIndex] = 1
          else:
              self.pwmStartBit[chIndex] = 0

          for i in range(8):
              status += (self.pwmStartBit[i] << i)

          ret = pac_api.pac_SetPWMStart(self.hPort, PAC_REMOTE_IO(int(self.txtAddress.get())), status)
          if not ret:
              ec = pac_api.pac_GetLastError()
              messagebox.showerror("Error", f"{PACNET.ErrCode(ec)}\nError Code: 0x{ec:X}")

if __name__ == "__main__":
    app = Form1()
    app.mainloop()

