import ctypes
from ctypes import c_bool, c_void_p, c_int, c_float, c_ulong, c_char_p, POINTER

##
pac_api = ctypes.CDLL('./PACSDK.dll')

#
PAC_REMOTE_IO = lambda iAddress: 0x1000 | iAddress

# 
pac_api.uart_Open.argtypes = [c_char_p]
pac_api.uart_Open.restype = c_void_p

pac_api.uart_Close.argtypes = [c_void_p]
pac_api.uart_Close.restype = c_bool

pac_api.pac_ReadAI.argtypes = [c_void_p, c_int, c_int, c_int, POINTER(c_float)]
pac_api.pac_ReadAI.restype = c_bool

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong

# 
def main():
    connection_string ="COM4,9600,N,8,1"
    #input('Input com\'s connection string (like "COM2,9600,N,8,1"): ')
    hPort = pac_api.uart_Open(connection_string.encode('utf-8'))
    
    if hPort == c_void_p(-1).value:  # INVALID_HANDLE_VALUE
        print("Open Error: Unable to open COM port.")
        return
    
    iAddr = int(input('Input address (hex): '), 16)
    iAIChannels = int(input('Input AI total channels: '))
    
    while True:
        print("\n1. Read AI value\n2. Exit")
        choice = int(input("Input number: "))
        
        if choice == 1:
            iChannel = int(input("Input channel to read AI: "))
            fAI_Value = c_float()
            err = pac_api.pac_ReadAI(hPort, PAC_REMOTE_IO(iAddr), iChannel, iAIChannels, ctypes.byref(fAI_Value))
            if err:
                print(f"The read AI's value is {fAI_Value.value}")
            else:
                ret= pac_api.pac_GetLastError()
                print("Error reading AI value.")
                print(hex(ret))
            input("Press any key to continue...")
        elif choice == 2:
            break
    
    err = pac_api.uart_Close(hPort)
    if not err:
        print("Close Error: Unable to close COM port.")

if __name__ == "__main__":
    main()

