import ctypes
from ctypes import *

# load PACSDK SDK
pac_api = ctypes.CDLL('.\\PACSDK.dll')

# 
PAC_REMOTE_IO = lambda iAddress: 0x1000 | iAddress

pac_api.uart_Close.argtypes = [c_void_p]
pac_api.uart_Close.restype = c_bool

pac_api.pac_WriteAO.argtypes = [c_void_p, c_int, c_int, c_int, c_float]
pac_api.pac_WriteAO.restype = c_bool

pac_api.pac_ReadAO.argtypes = [c_void_p, c_int, c_int, c_int, ctypes.POINTER(c_float)]
pac_api.pac_ReadAO.restype = c_bool

pac_api.pac_GetErrorMessage.argtypes = [c_ulong, c_char_p]
pac_api.pac_GetErrorMessage.restype = None

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong

# main
def main():
    connection_string = input("Input com's connectionstring (like 'COM2,9600,N,8,1'): ")
    hPort = pac_api.uart_Open(connection_string.encode('utf-8'))

    if hPort == c_void_p(-1).value:  # INVALID_HANDLE_VALUE
        strErr = ctypes.create_string_buffer(32)
        pac_api.pac_GetErrorMessage(pac_api.pac_GetLastError(), strErr)
        print(f"Open Error: {strErr.value.decode()}. The error code is {hex(pac_api.pac_GetLastError())}")
        return

    iAddr = int(input("Input address (hex): "), 16)
    iAOChannels = int(input("Input AO total channels: "))
    bQuit = False

    while not bQuit:
        print("\n1. Write AO value and Read AO value\n2. Exit")
        iNum = int(input("Input number: "))
        
        if iNum == 1:
            iChannel = int(input("Input channel to write AO: "))
            fAO_Value = float(input("Input AO value (float): "))

            # wriet AO value
            if pac_api.pac_WriteAO(hPort, PAC_REMOTE_IO(iAddr), iChannel, iAOChannels, fAO_Value):
                print(f"Write AO value {fAO_Value}")
                
                # read AO value
                read_value = c_float()
                if pac_api.pac_ReadAO(hPort, PAC_REMOTE_IO(iAddr), iChannel, iAOChannels, ctypes.byref(read_value)):
                    print(f"The read back AO's value is {read_value.value}")
                else:
                    strErr = ctypes.create_string_buffer(32)
                    pac_api.pac_GetErrorMessage(pac_api.pac_GetLastError(), strErr)
                    print(f"Read Error: {strErr.value.decode()}. The error code is {hex(pac_api.pac_GetLastError())}")

            else:
                strErr = ctypes.create_string_buffer(32)
                pac_api.pac_GetErrorMessage(pac_api.pac_GetLastError(), strErr)
                print(f"Write Error: {strErr.value.decode()}. The error code is {hex(pac_api.pac_GetLastError())}")

        elif iNum == 2:
            bQuit = True

    if not pac_api.uart_Close(hPort):
        strErr = ctypes.create_string_buffer(32)
        pac_api.pac_GetErrorMessage(pac_api.pac_GetLastError(), strErr)
        print(f"Close Error: {strErr.value.decode()}. The error code is {hex(pac_api.pac_GetLastError())}")

if __name__ == "__main__":
    main()

