import ctypes
from ctypes import *

# load PACSDK SDK
pac_api = ctypes.CDLL('.\\PACSDK.dll')

# 
PAC_REMOTE_IO = lambda iAddress: 0x1000 | iAddress

# 
pac_api.uart_Open.argtypes = [c_char_p]
pac_api.uart_Open.restype = c_void_p

pac_api.uart_Close.argtypes = [c_void_p]
pac_api.uart_Close.restype = c_bool

pac_api.pac_WriteDO.argtypes = [c_void_p, c_int, c_int, c_ulong]
pac_api.pac_WriteDO.restype = c_bool

pac_api.pac_ReadDIO.argtypes = [c_void_p, c_int, c_int, c_int, POINTER(c_ulong), POINTER(c_ulong)]
pac_api.pac_ReadDIO.restype = c_bool

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong

def get_last_error():
  error_code = pac_api.pac_GetLastError()
  error_message = create_string_buffer(64)
  pac_api.pac_GetErrorMessage(error_code, error_message)
  return error_message.value.decode()

def main():
  print("ICPDAS (R) SDK Demo for XP-8000")
  print("This demo is for remote 87K and 7K modules.")

  connection_string = input("Input com's connectionstring(like 'COM2,9600,N,8,1'): ")
  
 # open COM port
  hPort = pac_api.uart_Open(c_char_p(connection_string.encode('utf-8')))
  if hPort == ctypes.c_void_p(-1).value:
      print(f"Open Error: {get_last_error()}")
      return
  
  # Input address
  iAddr = int(input("Input address (hex): "), 16)
  
  # Inpute DO tatal channeles
  iDOChannels = int(input("Input DO total channels: "))
  # Inpute DI tatal channeles
  iDIChannels = int(input("Input DI total channels: "))

  while True:
      print("\n1. Write DO value\n2. Read DI and DO value\n3. Exit")
      iNum = int(input("Input number: "))
      
      if iNum == 1:
          lDO_Value = int(input("Input DO value (hex): "), 16)
          pac_api.pac_WriteDO(hPort, PAC_REMOTE_IO(iAddr), iDOChannels, lDO_Value)
          print(f"Write DO value 0x{lDO_Value:x}")
          input("Press any key to continue...")
      elif iNum == 2:
          lDI_Value = c_ulong()
          lDO_Value = c_ulong()
          err = pac_api.pac_ReadDIO(hPort, PAC_REMOTE_IO(iAddr), iDIChannels, iDOChannels, ctypes.byref(lDI_Value), ctypes.byref(lDO_Value))
          if err:
              print(f"The DI's value is {lDI_Value.value}")
              print(f"The DO's value is {lDO_Value.value}")
          else:
              print(f"Read Error: {get_last_error()}")
          input("Press any key to continue...")
      elif iNum == 3:
          break

  # close COM port
  if not pac_api.uart_Close(hPort):
      print(f"Close Error: {get_last_error()}")

if __name__ == "__main__":
  main()
