import tkinter as tk
from tkinter import ttk, messagebox
import ctypes
from ctypes import *

# Load the pacsdk.dll
pac_api = ctypes.CDLL('./PACSDK.dll')

# Define UART and AI Read functions (prototypes for the pacsdk functions)
pac_api.uart_Open.argtypes = [c_char_p]
pac_api.uart_Open.restype = c_void_p

pac_api.uart_Close.argtypes = [c_void_p]

pac_api.pac_SetPWMDuty.argtypes = [c_void_p, c_int, c_short, c_float]
pac_api.pac_SetPWMDuty.restype = c_bool

pac_api.pac_GetPWMDuty.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_float)]
pac_api.pac_GetPWMDuty.restype = c_bool

pac_api.pac_SetPWMFrequency.argtypes = [c_void_p, c_int, c_short, c_ulong]
pac_api.pac_SetPWMFrequency.restype = c_bool

pac_api.pac_GetPWMFrequency.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_ulong)]
pac_api.pac_GetPWMFrequency.restype = c_bool

pac_api.pac_SetPWMMode.argtypes = [c_void_p, c_int, c_short, c_long]
pac_api.pac_SetPWMMode.restype = c_bool

pac_api.pac_GetPWMMode.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_long)]
pac_api.pac_GetPWMMode.restype = c_bool

pac_api.pac_SetPWMStart.argtypes = [c_void_p, c_int, c_short]
pac_api.pac_SetPWMStart.restype = c_bool

pac_api.pac_SavePWMConfig.argtypes = [c_void_p, c_int]
pac_api.pac_SavePWMConfig.restype = c_bool

pac_api.pac_GetPWMDIOStatus.argtypes = [c_void_p, c_int, ctypes.POINTER(c_ubyte), ctypes.POINTER(c_ubyte)]
pac_api.pac_GetPWMDIOStatus.restype = c_bool

pac_api.pac_SetPWMPulseCount.argtypes = [c_void_p, c_int, c_short, c_long]
pac_api.pac_SetPWMPulseCount.restype = c_bool

pac_api.pac_GetPWMPulseCount.argtypes = [c_void_p, c_int, c_short, ctypes.POINTER(c_long)]
pac_api.pac_GetPWMPulseCount.restype = c_bool

pac_api.pac_GetLastError.argtypes = []
pac_api.pac_GetLastError.restype = c_ulong

# Tkinter GUI setup
class Form1(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("7K87K PWM")
        self.geometry("800x600")  # Window size


        # Assuming a base width of each widget (adjust as needed)
        base_width = 100
        base_height = 25  # Assuming consistent height for most widgets

        # Calculate adjusted x-positions with spacing
        x_left_column = 36  # Start of left column (Frequency label)
        x_center_column = x_left_column + base_width + 20  # Center column (Duty label)
        x_center_column_2= x_center_column + base_width + 20  # Center column (Burst count label)
         x_center_column_3= x_center_column_2 + base_width + 20  # Center column (Apply PWM Settings)
        x_right_column_1 = x_center_column_3 + base_width + 30  # Right column (DI Status)
        x_right_column_2 = x_right_column_1 + base_width + 30  # Right column (PWM Status)

        # Calculate starting y-position for the first row
        y_pos = 125

        # Create and place widgets

        # Top row labels
        self.label3 = tk.Label(self, text="PWM Pulse Mode")
        self.label3.place(x=x_center_column, y=y_pos, width=120, height=base_height)

        self.label5 = tk.Label(self, text="Frequency (Hz)")
        self.label5.place(x=x_left_column, y=y_pos, width=100, height=base_height)

        self.label7 = tk.Label(self, text="Duty")
        self.label7.place(x=x_center_column, y=y_pos, width=50, height=base_height)

        # "Burst Count" label
        self.label19 = tk.Label(self, text="Burst Count")
        self.label19.place(x=x_center_column_2, y=y_pos, width=50, height=base_height)

        # Channel labels
        self.channel = [tk.Label(self, text=f"CH{i}:") for i in range(8)]
        for i, lb in enumerate(self.channel):
            lb.place(x=1, y=145 + i * 35, width=40, height=base_height)

        # DI Status Labels
        self.lbDI = [tk.Label(self, text="OFF") for _ in range(8)]
        for i, lb in enumerate(self.lbDI):
            lb.place(x=x_right_column_1 + 10, y=y_pos + i * 35, width=40, height=base_height)  # Added spacing

        self.label18 = tk.Label(self, text="DI Status")
        self.label18.place(x=x_right_column_1, y=y_pos, width=80, height=base_height)

        # PWM Status Labels
        self.pwm = [tk.Label(self, text="OFF") for _ in range(8)]
        for i, lb in enumerate(self.pwm):
            lb.place(x=x_right_column_2 + 10, y=y_pos + i * 35, width=40, height=base_height)  # Added spacing

        self.label20 = tk.Label(self, text="PWM Status")
        self.label20.place(x=x_right_column_2, y=y_pos, width=80, height=base_height)

        # Duty, Frequency, and Mode entries (adjusted positions)
        self.txtDuty = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtDuty):
            entry.place(x=x_center_column, y=y_pos + i * 35, width=50, height=base_height)
            entry.insert(0, "50")

        self.txtHz = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtHz):
            entry.place(x=x_left_column, y=y_pos + i * 35, width=80, height=base_height)
            entry.insert(0, "1000")

        self.txtCnt = [tk.Entry(self) for _ in range(8)]
        for i, entry in enumerate(self.txtCnt):
            entry.place(x=x_center_column_2, y=y_pos + i * 35, width=80, height=30)  # Increased width and height
            entry.insert(0, "10000")
            
        self.cboMode = [ttk.Combobox(self, values=["Burst Count", "Continuous"]) for _ in range(8)]
        for i, combo in enumerate(self.cboMode):
            combo.place(x=x_center_column + base_width + 10, y=y_pos + i * 35, width=120, height=base_height)

        # Button and Checkbutton positions (adjusted)
        self.btnPWM = [tk.Button(self, text="Apply PWM Settings", command=lambda i=i: self.ConfigPWMSettings(i)) for i in range(8)]
        for i, btn in enumerate(self.btnPWM):
            btn.place(x=x_center_column_3+ base_width + 10, y=y_pos + i * 35, width=120, height=base_height)

if __name__ == "__main__":
    app = Form1()
    app.mainloop()

